/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
﻿namespace Phoenix.Translation.Test
{
    using System;
    using System.Data;
    using System.Data.SQLite;
    using System.IO;
    using System.Reflection;
    using System.Text.RegularExpressions;

    using Phoenix.Tools.Translation;

    /// <summary>
    /// Test suite for translation
    /// </summary>
    class TestTranslation
    {
        #region Fields

        private string connectionString;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ConfigureAndTestTranslation"/> class.
        /// </summary>
        public TestTranslation(string connectionString)
        {
            this.connectionString = connectionString;
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Removes the DB.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns></returns>
        public static bool RemoveDB(string connectionString)
        {
            bool ret = false;
            Regex regex = new Regex("Data Source=(?<path>.*);Version=3;");
            Match match = regex.Match(connectionString);
            string file = match.Groups["path"].Value;

            if (File.Exists(file))
            {
                ret = true;
                File.Delete(file);
            }

            return ret;
        }

        /// <summary>
        /// Configures the and test translation assembly.
        /// </summary>
        public void ConfigureAndTestTranslation()
        {
            Translator.Load(SQLiteRepository.Create());

            Console.WriteLine("");
            Translator.SetLanguage(Language.EN);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());

            Console.WriteLine("");
            Translator.SetLanguage(Language.ES);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());

            Console.WriteLine("");
            Translator.SetLanguage(Language.FR);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());
        }

        /// <summary>
        /// Creates or recreate a test database for translation.
        /// </summary>
        public void CreateTestDatabase()
        {
            //if (File.Exists(connectionString))
            //    File.Delete(connectionString);

            using (SQLiteConnection connection = new SQLiteConnection(connectionString))
            using (SQLiteCommand command = new SQLiteCommand(connection))
            {
                command.CommandType = CommandType.Text;
                command.CommandText = this.GetDdlCode();
                connection.Open();
                command.ExecuteNonQuery();
                connection.Close();
            }
        }

        /// <summary>
        /// Tests the translation. If called bedore <see cref="ConfigureAndTestTranslation"/> should'nt translate.
        /// Otherwise works the same way than <see cref="ConfigureAndTestTranslation"/>
        /// </summary>
        public void OnlyTestTranslation()
        {
            Console.WriteLine("");
            Translator.SetLanguage(Language.EN);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());

            Console.WriteLine("");
            Translator.SetLanguage(Language.ES);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());

            Console.WriteLine("");
            Translator.SetLanguage(Language.FR);
            Console.WriteLine("Hello".Translate());
            Console.WriteLine("What is your name?".Translate());
            Console.WriteLine("My name is {0}.".Translate("Jean-Baptiste"));
            Console.WriteLine("How are you?".Translate());
        }

        /// <summary>
        /// Tests the automatic add if translation does not exist.
        /// </summary>
        /// <returns></returns>
        public bool TestAutomaticAdd()
        {
            long countBefore = this.CountTranslation();
            string newValue = Guid.NewGuid().ToString();
            newValue.Translate();
            long countAfter = this.CountTranslation();

            return ((countBefore - countAfter) < 0);
        }

        private long CountTranslation()
        {
            string query = "SELECT COUNT(*) FROM BaseText;";
            object count;

            string path = Assembly.GetExecutingAssembly().Location;
            string asmName = typeof(TestTranslation).Assembly.GetName().Name + ".exe";
            path = Path.Combine(path.Replace(asmName, ""), "translation.db");

            string connectionString = string.Format("Data Source={0};Version=3;", path);

            using (SQLiteConnection connection = new SQLiteConnection(connectionString))
            using (SQLiteCommand command = new SQLiteCommand(connection))
            {
                command.CommandType = CommandType.Text;
                command.CommandText = query;
                connection.Open();
                count = command.ExecuteScalar();
                connection.Close();
            }
            return (long)count;
        }

        /// <summary>
        /// Gets the DDL code.
        /// </summary>
        /// <returns>SQL query to create the test database.</returns>
        private string GetDdlCode()
        {
            string assemblyName = typeof(TestTranslation).Assembly.GetName().Name;
            string fileName = "Resources.testdb.sql";

            try
            {
                Assembly assembly = Assembly.Load(assemblyName);
                using (Stream str = assembly.GetManifestResourceStream(assemblyName + "." + fileName))
                {

                    if (str == null)
                        throw new ArgumentNullException("str", "Could not locate embedded resource '" + fileName + "' in assembly '" + assemblyName + "'");
                    return new StreamReader(str).ReadToEnd();
                }
            }
            catch (Exception e)
            {
                throw new Exception(assemblyName + ": " + e.Message);
            }
        }

        #endregion Methods
    }
}